'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _svg2png = require('svg2png');

var _svg2png2 = _interopRequireDefault(_svg2png);

var _faviconGenerator = require('./favicon-generator.js');

var _faviconGenerator2 = _interopRequireDefault(_faviconGenerator);

var _icnsGenerator = require('./icns-generator.js');

var _icnsGenerator2 = _interopRequireDefault(_icnsGenerator);

var _icoGenerator = require('./ico-generator.js');

var _icoGenerator2 = _interopRequireDefault(_icoGenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Generate the PNG files = require(SVG file.
 */
class PNGGenerator {
  /**
   * Generate the PNG files = require(the SVG file.
   *
   * @param {String}         src    SVG file path.
   * @param {String}         dir    Output destination The path of directory.
   * @param {Array.<String>} modes  Modes of an output files.
   * @param {Function}       cb     Callback function.
   * @param {Logger}         logger Logger.
   */
  static generate(src, dir, modes, cb, logger) {
    _fs2.default.readFile(src, (err, svg) => {
      if (err) {
        cb(err);
        return;
      }

      logger.log('SVG to PNG:');

      const sizes = PNGGenerator.getRequiredImageSizes(modes);
      Promise.all(sizes.map(size => {
        return PNGGenerator._generatePNG(svg, size, dir, logger);
      })).then(results => {
        cb(null, results);
      }).catch(err2 => {
        cb(err2);
      });
    });
  }

  /**
   * Gets the size of the images needed to create an icon.
   *
   * @param {Array.<String>} modes Modes of an output files.
   *
   * @return {Array.<Number>} The sizes of the image.
   */
  static getRequiredImageSizes(modes) {
    let sizes = [];
    if (modes && 0 < modes.length) {
      modes.forEach(mode => {
        switch (mode) {
          case 'icns':
            sizes = sizes.concat(_icnsGenerator2.default.getRequiredImageSizes());
            break;

          case 'ico':
            sizes = sizes.concat(_icoGenerator2.default.getRequiredImageSizes());
            break;

          case 'favicon':
            sizes = sizes.concat(_faviconGenerator2.default.getRequiredImageSizes());
            break;

          default:
            break;
        }
      });
    }

    // 'all' mode
    if (sizes.length === 0) {
      sizes = _faviconGenerator2.default.getRequiredImageSizes().concat(_icnsGenerator2.default.getRequiredImageSizes().concat(_icoGenerator2.default.getRequiredImageSizes()));
    }

    return sizes.filter((value, index, array) => {
      return array.indexOf(value) === index;
    }).sort((a, b) => {
      // Always ensure the ascending order
      return a - b;
    });
  }

  /**
   * Generate the PNG file = require(the SVG data.
   *
   * @param {Buffer} svg    SVG data that has been parse by svg2png.
   * @param {Number} size   The size (width/height) of the image.
   * @param {String} dir    Path of the file output directory.
   * @param {Logger} logger Logger.
   *
   * @return {Promise} Image generation task.
   */
  static _generatePNG(svg, size, dir, logger) {
    return new Promise((resolve, reject) => {
      if (!(svg && 0 < size && dir)) {
        reject(new Error('Invalid parameters.'));
        return;
      }

      const dest = _path2.default.join(dir, size + '.png');
      logger.log('  Create: ' + dest);

      const buffer = _svg2png2.default.sync(svg, { width: size, height: size });
      if (!buffer) {
        reject(new Error('Faild to write the image, ' + size + 'x' + size));
        return;
      }

      _fs2.default.writeFile(dest, buffer, err => {
        if (err) {
          reject(err);
          return;
        }

        resolve({ size: size, path: dest });
      });
    });
  }
}
exports.default = PNGGenerator;